package controllers

import (
	"hongze/hongze_admin/models"
	"hongze/hongze_admin/services"
	"hongze/hongze_admin/utils"
	"os"
	"path"
	"time"
)

//音频
type VoiceController struct {
	BaseAuthController
}

// @Title 音频上传
// @Description 音频上传接口
// @Param   file   query   file  true       "文件"
// @Param   ReportId   query   int  true       "报告ID"
// @Success Ret=200 上传成功
// @router /upload [post]
func (this *VoiceController) Upload() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	f, h, err := this.GetFile("file")
	if err != nil {
		br.Msg = "获取资源信息失败"
		br.ErrMsg = "获取资源信息失败,Err:" + err.Error()
		return
	}
	reportId, err := this.GetInt("ReportId")
	if err != nil {
		br.Msg = "获取资源信息失败"
		br.ErrMsg = "获取资源信息失败,Err:" + err.Error()
		return
	}
	report, err := models.GetReportItemById(reportId)
	if err != nil {
		br.Msg = "获取报告信息失败"
		br.ErrMsg = "获取报告信息失败,Err:" + err.Error()
		return
	}

	ext := path.Ext(h.Filename)
	dateDir := time.Now().Format("20060102")
	uploadDir := utils.STATIC_DIR + "hongze/" + dateDir
	err = os.MkdirAll(uploadDir, 777)
	if err != nil {
		br.Msg = "存储目录创建失败"
		br.ErrMsg = "存储目录创建失败,Err:" + err.Error()
		return
	}
	randStr := utils.GetRandStringNoSpecialChar(28)
	fileName := randStr + ext
	fpath := uploadDir + "/" + fileName
	defer f.Close() //关闭上传文件
	err = this.SaveToFile("file", fpath)
	if err != nil {
		br.Msg = "文件上传失败"
		br.ErrMsg = "文件上传失败，Err:" + err.Error()
		return
	}
	//上传到阿里云
	resourceUrl, err := services.UploadAudioAliyun(fileName, fpath)
	if err != nil {
		br.Msg = "文件上传失败"
		br.ErrMsg = "文件上传失败，Err:" + err.Error()
		return
	}
	defer func() {
		os.Remove(fpath)
	}()
	item := new(models.Resource)
	item.ResourceUrl = resourceUrl
	item.ResourceType = 2
	item.CreateTime = time.Now()
	newId, err := models.AddResource(item)
	if err != nil {
		br.Msg = "资源上传失败"
		br.ErrMsg = "资源上传失败,Err:" + err.Error()
		return
	}
	playSeconds, err := utils.GetVideoPlaySeconds(fpath)
	if err != nil {
		br.Msg = "获取音频时间失败"
		br.ErrMsg = "获取音频时间失败,Err:" + err.Error()
		return
	}
	createTime := report.CreateTime.Format("0102")
	videoName := report.Title + "(" + createTime + ")"
	err = models.ModifyReportVideo(reportId, resourceUrl, videoName, playSeconds)
	if err != nil {
		br.Msg = "上传失败"
		br.ErrMsg = "上传失败，Err:" + err.Error()
		return
	}
	resp := new(models.ResourceResp)
	resp.Id = newId
	resp.ResourceUrl = resourceUrl
	br.Msg = "上传成功"
	br.Ret = 200
	br.Success = true
	br.Data = resp
	return
}
